==> schedwiclnt-0.1.0.tar.gz: schedwiclnt-0.1.0/src/utils.c <==
/* Schedwi
   Copyright (C) 2007 Herve Quatremain

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Library General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/* utils.c -- Several functions */

#include <schedwi.h>

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#else
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#endif
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#if HAVE_SIGNAL_H
#include <signal.h>
#endif

#if HAVE_FCNTL_H
#include <fcntl.h>
#endif

#if HAVE_ASSERT_H
#include <assert.h>
#endif

#if HAVE_ERRNO_H
#include <errno.h>
#endif
#ifndef errno
extern int errno;
#endif

#include <lib_functions.h>
#include <utils.h>

#if _GNU_SOURCE && !HAVE_CLEARENV
extern char **environ;
#endif


/*
 * Return the filename part of a path
 *
 * Return:
 *   A pointer to the filename part in the provided path
 */
const char *
base_name (const char *path)
{
	const char * s, * prev;
	size_t sep_length;

#ifdef HAVE_ASSERT_H
	assert (path != 0);
#endif
	prev = 0;
	s    = path;
	sep_length = schedwi_strlen (DIR_SEP);
	while ((s = strstr (s, DIR_SEP)) != 0) {
		s += sep_length;
		prev = s;
	}
	if (prev == 0) {
		return path;
	}
	return prev;
}


/*
 * Print copyright and version information to stdout
 */
void
version (const char *prog_name)
{
	printf ("%s (Schedwi) %s\n", prog_name, PACKAGE_VERSION);
	puts("Copyright (C) 2007 Herve Quatremain");
	puts("Schedwi is free software, covered by the GNU");
	puts("General Public License version 2 or later.");
	puts("You may redistribute copies of Schedwi under");
	puts("the terms of this license.");
}


/*
 * Remove a file. If the remove fails, then try to truncate (size=0) the
 * file
 *
 * Return:
 *   0 --> File removed or truncated
 *  -1 --> Error (errno is set)
 */
int
my_unlink (const char *file_name)
{
	FILE *f;
	int save_errno;

	if (file_name == NULL || file_name[0] == '\0') {
		return 0;
	}

	/* Remove the file */
	if (unlink (file_name) != 0) {

		/* File does not exist */
		if (errno == ENOENT) {
			return 0;
		}

		/* Can't remove the file.  Try to truncate it */
		save_errno = errno;
		f = fopen (file_name, "wb");
		if (f == NULL) {
			/* Fail to truncate the file.  Return an error */
			errno = save_errno;
			return -2;
		}
		fclose (f);
	}
	return 0;
}


/*
 * RAZ the process structures:
 *    - Close all open files (except the provided file descriptor)
 *    - Empty the environment
 *    - Set the default action for all the signals
 *    - Remove the signal mask
 *
 * Failure of one of this step is not reported
 *
 * Note:
 *    This function is called in two places by the Schedwi agent (schedwiclnt):
 *       - At the start of the agent, when it goes into background
 *       - Every time a job is started
 *
 *    Clear the environment may cause issues.  In particular, some variables
 *    may be required by the job subprocess.  For instance, using Cygwin, the
 *    PATH environment variable contains the path to the Cygwin DLL
 *    (cygwin1.dll) which is required to fork/exec the job command.  This is
 *    why the environment is not cleared for Cygwin only.
 *    On other systems, other variables may be required (LD_LIBRARY_PATH for
 *    instance for SunOS and Linux) but no special care is taken and they
 *    will be erased by this function.  If they are required they can be
 *    defined as an environment for the job in the Schedwi server.
 */
void
sanitise (int fd)
{
	int i;
	struct sigaction sa;
	sigset_t mask;

	/* Close all files (except fd) */
	for (i = getdtablesize () - 1; i >= 0; i--) {
		if (i != fd) {
			close (i);
		}
	}

	/* Clear the environment */
#ifndef CYGWIN_IN_USE
#if HAVE_CLEARENV
	clearenv ();
#elif _GNU_SOURCE
	environ = NULL;
#endif
#endif

	/* Clear the signal actions and mask */
	schedwi_memset (&sa, 0, sizeof (struct sigaction));
	sa.sa_handler = SIG_DFL;
	sigemptyset (&sa.sa_mask);
	for (i = 0; i < 128; i++) {
		sigaction (i, &sa, NULL);
	}
	sigfillset (&mask);
	sigprocmask (SIG_UNBLOCK, &mask, NULL);
}


/*
 * Clean the process
 */
void
clean_process ()
{
#if HAVE_GETENV && HAVE_CHDIR
	char *tmp;
#endif

	/* Change to a temporary directory */
	tmp = NULL;
#if HAVE_CHDIR
#if HAVE_GETENV
	tmp = getenv ("TMPDIR");
	if (tmp != NULL) {
		if (chdir (tmp) != 0) {
			tmp = NULL;
		}
	}
#endif /* HAVE_GETENV */
	if (tmp == NULL) {
#ifdef P_tmpdir
		chdir (P_tmpdir);
#else
		chdir ("/tmp");
#endif
	}
#endif /* HAVE_CHDIR */

	/* Clean the process */
	sanitise (-1);

#if HAVE_UMASK
	umask (0027);
#endif
}


/*
 * Start the daemon
 *
 * Return:
 *   0 --> No error
 *  -1 --> fork error (errno is set)
 */
int
go_daemon()
{
#if HAVE_PID_T
	pid_t p;
#else
	int p;
#endif

	p = fork ();
	if (p < 0) {
		return -1;
	}
	if (p > 0) {
		exit (0);
	}

#if HAVE_SETSID
	setsid();
#endif

	p = fork ();
	if (p < 0) {
		return -1;
	}
	if (p > 0) {
		exit (0);
	}

	clean_process ();
	return 0;
}


static int fd_pid_file = -1;

/*
 * Write the PID of the current process to the provided file.
 *
 * Return:
 *   0 --> No error
 *  -1 --> Fail to open the file (errno is set)
 *   1 --> An other daemon is already running
 */
int
write_pid_file (const char *file_name)
{
	char pid_str[26];
	size_t l;

	if (file_name == NULL) {
		return 0;
	}

	/* Open the PID file */
	fd_pid_file = open (file_name, O_RDWR|O_CREAT, 00644);
	if (fd_pid_file < 0) {
		return -1;
	}

	/*
	 * Try to lock it.  If it fails it's probably because an other
	 * daemon is already running
	 */
#if HAVE_LOCKF && F_TLOCK
	if (lockf (fd_pid_file, F_TLOCK, 0) < 0) {
		return 1;
	}
#elif HAVE_FLOCK && LOCK_EX && LOCK_NB
	if (flock (fd_pid_file, LOCK_EX | LOCK_NB) < 0) {
		return 1;
	}
#endif

	/* Write the PID to the file */
	l = snprintf (pid_str, 26, "%ld\n", (long int)getpid ());
	pid_str [26 - 1] = '\0';
	write (fd_pid_file, pid_str, schedwi_strlen (pid_str));
	return 0;
}


/*
 * Close and remove the file containing the PID of the daemon
 */
void
close_pid_file (const char *file_name)
{
	if (file_name != NULL) {
		my_unlink (file_name);
	}

	if (fd_pid_file >= 0) {
		close (fd_pid_file);
	}
	fd_pid_file = -1;
}

/*------------------------======= End Of File =======------------------------*/

==> schedwiclnt-0.1.1.tar.gz: schedwiclnt-0.1.1/src/utils.c <==
/* Schedwi
   Copyright (C) 2007 Herve Quatremain
     
   This file is part of Schedwi.

   Schedwi is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   Schedwi is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* utils.c -- Several functions */

#include <schedwi.h>

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#else
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#endif
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#if HAVE_SIGNAL_H
#include <signal.h>
#endif

#if HAVE_FCNTL_H
#include <fcntl.h>
#endif

#if HAVE_ASSERT_H
#include <assert.h>
#endif

#if HAVE_ERRNO_H
#include <errno.h>
#endif
#ifndef errno
extern int errno;
#endif

#include <lib_functions.h>
#include <utils.h>

#if _GNU_SOURCE && !HAVE_CLEARENV
extern char **environ;
#endif


/*
 * Return the filename part of a path
 *
 * Return:
 *   A pointer to the filename part in the provided path
 */
const char *
base_name (const char *path)
{
	const char * s, * prev;
	size_t sep_length;

#ifdef HAVE_ASSERT_H
	assert (path != 0);
#endif
	prev = 0;
	s    = path;
	sep_length = schedwi_strlen (DIR_SEP);
	while ((s = strstr (s, DIR_SEP)) != 0) {
		s += sep_length;
		prev = s;
	}
	if (prev == 0) {
		return path;
	}
	return prev;
}


/*
 * Print copyright and version information to stdout
 */
void
version (const char *prog_name)
{
	printf ("%s (Schedwi) %s\n", prog_name, PACKAGE_VERSION);
	puts("Copyright (C) 2007 Herve Quatremain");
	puts("Schedwi is free software, covered by the GNU");
	puts("General Public License version 2 or later.");
	puts("You may redistribute copies of Schedwi under");
	puts("the terms of this license.");
}


/*
 * Remove a file. If the remove fails, then try to truncate (size=0) the
 * file
 *
 * Return:
 *   0 --> File removed or truncated
 *  -1 --> Error (errno is set)
 */
int
my_unlink (const char *file_name)
{
	FILE *f;
	int save_errno;

	if (file_name == NULL || file_name[0] == '\0') {
		return 0;
	}

	/* Remove the file */
	if (unlink (file_name) != 0) {

		/* File does not exist */
		if (errno == ENOENT) {
			return 0;
		}

		/* Can't remove the file.  Try to truncate it */
		save_errno = errno;
		f = fopen (file_name, "wb");
		if (f == NULL) {
			/* Fail to truncate the file.  Return an error */
			errno = save_errno;
			return -2;
		}
		fclose (f);
	}
	return 0;
}


/*
 * RAZ the process structures:
 *    - Close all open files (except the provided file descriptor)
 *    - Empty the environment
 *    - Set the default action for all the signals
 *    - Remove the signal mask
 *
 * Failure of one of this step is not reported
 *
 * Note:
 *    This function is called in two places by the Schedwi agent (schedwiclnt):
 *       - At the start of the agent, when it goes into background
 *       - Every time a job is started
 *
 *    Clear the environment may cause issues.  In particular, some variables
 *    may be required by the job subprocess.  For instance, using Cygwin, the
 *    PATH environment variable contains the path to the Cygwin DLL
 *    (cygwin1.dll) which is required to fork/exec the job command.  This is
 *    why the environment is not cleared for Cygwin only.
 *    On other systems, other variables may be required (LD_LIBRARY_PATH for
 *    instance for SunOS and Linux) but no special care is taken and they
 *    will be erased by this function.  If they are required they can be
 *    defined as an environment for the job in the Schedwi server.
 */
void
sanitise (int fd)
{
	int i;
	struct sigaction sa;
	sigset_t mask;

	/* Close all files (except fd) */
	for (i = getdtablesize () - 1; i >= 0; i--) {
		if (i != fd) {
			close (i);
		}
	}

	/* Clear the environment */
#ifndef CYGWIN_IN_USE
#if HAVE_CLEARENV
	clearenv ();
#elif _GNU_SOURCE
	environ = NULL;
#endif
#endif

	/* Clear the signal actions and mask */
	schedwi_memset (&sa, 0, sizeof (struct sigaction));
	sa.sa_handler = SIG_DFL;
	sigemptyset (&sa.sa_mask);
	for (i = 0; i < 128; i++) {
		sigaction (i, &sa, NULL);
	}
	sigfillset (&mask);
	sigprocmask (SIG_UNBLOCK, &mask, NULL);
}


/*
 * Clean the process
 */
void
clean_process ()
{
#if HAVE_GETENV && HAVE_CHDIR
	char *tmp;
#endif

	/* Change to a temporary directory */
	tmp = NULL;
#if HAVE_CHDIR
#if HAVE_GETENV
	tmp = getenv ("TMPDIR");
	if (tmp != NULL) {
		if (chdir (tmp) != 0) {
			tmp = NULL;
		}
	}
#endif /* HAVE_GETENV */
	if (tmp == NULL) {
#ifdef P_tmpdir
		chdir (P_tmpdir);
#else
		chdir ("/tmp");
#endif
	}
#endif /* HAVE_CHDIR */

	/* Clean the process */
	sanitise (-1);

#if HAVE_UMASK
	umask (0027);
#endif
}


/*
 * Start the daemon
 *
 * Return:
 *   0 --> No error
 *  -1 --> fork error (errno is set)
 */
int
go_daemon()
{
#if HAVE_PID_T
	pid_t p;
#else
	int p;
#endif

	p = fork ();
	if (p < 0) {
		return -1;
	}
	if (p > 0) {
		exit (0);
	}

#if HAVE_SETSID
	setsid();
#endif

	p = fork ();
	if (p < 0) {
		return -1;
	}
	if (p > 0) {
		exit (0);
	}

	clean_process ();
	return 0;
}


static int fd_pid_file = -1;

/*
 * Write the PID of the current process to the provided file.
 *
 * Return:
 *   0 --> No error
 *  -1 --> Fail to open the file (errno is set)
 *   1 --> An other daemon is already running
 */
int
write_pid_file (const char *file_name)
{
	char pid_str[26];
	size_t l;

	if (file_name == NULL) {
		return 0;
	}

	/* Open the PID file */
	fd_pid_file = open (file_name, O_RDWR|O_CREAT, 00644);
	if (fd_pid_file < 0) {
		return -1;
	}

	/*
	 * Try to lock it.  If it fails it's probably because an other
	 * daemon is already running
	 */
#if HAVE_LOCKF && F_TLOCK
	if (lockf (fd_pid_file, F_TLOCK, 0) < 0) {
		return 1;
	}
#elif HAVE_FLOCK && LOCK_EX && LOCK_NB
	if (flock (fd_pid_file, LOCK_EX | LOCK_NB) < 0) {
		return 1;
	}
#endif

	/* Write the PID to the file */
	l = snprintf (pid_str, 26, "%ld\n", (long int)getpid ());
	pid_str [26 - 1] = '\0';
	write (fd_pid_file, pid_str, schedwi_strlen (pid_str));
	return 0;
}


/*
 * Close and remove the file containing the PID of the daemon
 */
void
close_pid_file (const char *file_name)
{
	if (file_name != NULL) {
		my_unlink (file_name);
	}

	if (fd_pid_file >= 0) {
		close (fd_pid_file);
	}
	fd_pid_file = -1;
}

/*------------------------======= End Of File =======------------------------*/
